""" Hotfix for CMF Collector #259.

$Id: __init__.py,v 1.3 2004/08/08 00:05:54 tseaver Exp $
"""
from cgi import escape

from AccessControl import getSecurityManager
from AccessControl.Permissions import delete_objects as DeleteObjects
from Acquisition import aq_base, aq_inner, aq_parent
from App.Dialogs import MessageDialog
from OFS.CopySupport import CopyError
from OFS.CopySupport import eInvalid
from OFS.CopySupport import eNoData
from OFS.CopySupport import eNotFound
from OFS.CopySupport import eNotSupported
from OFS.CopySupport import _cb_decode
from OFS.CopySupport import sanity_check
from OFS.CopySupport import cookie_path
from OFS.CopySupport import absattr
from OFS.Moniker import  loadMoniker
from zExceptions import Unauthorized

try:
    from Products.CMFCore.PortalFolder import PortalFolder
except ImportError:
    PortalFolder = None
else:
    from Products.CMFCore.utils import getToolByName


def _patched_manage_pasteObjects(self, cb_copy_data=None, REQUEST=None):
    """Paste previously copied objects into the current object.

    o If calling manage_pasteObjects from python code, pass
      the result of a previous call to manage_cutObjects or
      manage_copyObjects as the first argument.
    """
    cp=None
    if cb_copy_data is not None:
        cp=cb_copy_data
    else:
        if REQUEST and REQUEST.has_key('__cp'):
            cp=REQUEST['__cp']
    if cp is None:
        raise CopyError, eNoData

    try:    cp=_cb_decode(cp)
    except: raise CopyError, eInvalid

    oblist=[]
    op=cp[0]
    app = self.getPhysicalRoot()
    result = []

    for mdata in cp[1]:
        m = loadMoniker(mdata)
        try: ob = m.bind(app)
        except: raise CopyError, eNotFound
        self._verifyObjectPaste(ob, validate_src=op+1)
        oblist.append(ob)

    if op==0:
        # Copy operation
        for ob in oblist:
            if not ob.cb_isCopyable():
                raise CopyError, eNotSupported % escape(ob.getId())
            try:    ob._notifyOfCopyTo(self, op=0)
            except: raise CopyError, MessageDialog(
                title='Copy Error',
                message=sys.exc_info()[1],
                action ='manage_main')
            ob=ob._getCopy(self)
            orig_id=ob.getId()
            id=self._get_id(ob.getId())
            result.append({'id':orig_id, 'new_id':id})
            ob._setId(id)
            self._setObject(id, ob)
            ob = self._getOb(id)
            ob.manage_afterClone(ob)
            ob.wl_clearLocks()

        if REQUEST is not None:
            return self.manage_main(self, REQUEST, update_menu=1,
                                    cb_dataValid=1)

    if op==1:
        # Move operation
        for ob in oblist:
            id=ob.getId()
            if not ob.cb_isMoveable():
                raise CopyError, eNotSupported % escape(id)
            try:    ob._notifyOfCopyTo(self, op=1)
            except: raise CopyError, MessageDialog(
                title='Move Error',
                message=sys.exc_info()[1],
                action ='manage_main')
            if not sanity_check(self, ob):
                raise CopyError, 'This object cannot be pasted into itself'

            # try to make ownership explicit so that it gets carried
            # along to the new location if needed.
            ob.manage_changeOwnershipType(explicit=1)

            aq_parent(aq_inner(ob))._delObject(id)
            ob = aq_base(ob)
            orig_id=id
            id=self._get_id(id)
            result.append({'id':orig_id, 'new_id':id })
            ob._setId(id)

            self._setObject(id, ob, set_owner=0)

            # try to make ownership implicit if possible
            ob=self._getOb(id)
            ob.manage_changeOwnershipType(explicit=0)

        if REQUEST is not None:
            REQUEST['RESPONSE'].setCookie('__cp', 'deleted',
                                path='%s' % cookie_path(REQUEST),
                                expires='Wed, 31-Dec-97 23:59:59 GMT')
            REQUEST['__cp'] = None
            return self.manage_main(self, REQUEST, update_menu=1,
                                    cb_dataValid=0)
    return result

def _patched_CopySupport_verifyObjectPaste(self, object, validate_src=1):
    # Verify whether the current user is allowed to paste the
    # passed object into self. This is determined by checking
    # to see if the user could create a new object of the same
    # meta_type of the object passed in and checking that the
    # user actually is allowed to access the passed in object
    # in its existing context.
    #
    # Passing a false value for the validate_src argument will skip
    # checking the passed in object in its existing context. This is
    # mainly useful for situations where the passed in object has no
    # existing context, such as checking an object during an import
    # (the object will not yet have been connected to the acquisition
    # heirarchy).
    if not hasattr(object, 'meta_type'):
        raise CopyError, MessageDialog(
                title   = 'Not Supported',
                message = ('The object <EM>%s</EM> does not support this' \
                            ' operation' % object_id),
                action  = 'manage_main')

    if not hasattr(self, 'all_meta_types'):
        raise CopyError, MessageDialog(
                title   = 'Not Supported',
                message = 'Cannot paste into this object.',
                action  = 'manage_main')

    parent = aq_parent(aq_inner(object))
    object_id = escape(absattr(object.id))
    method_name = None
    mt_permission = None
    meta_types = absattr(self.all_meta_types)

    for d in meta_types:
        if d['name'] == object.meta_type:
            method_name = d['action']
            mt_permission = d.get('permission')
            break

    if method_name:
        try:
            method = self.restrictedTraverse(method_name)
            # method_name is e.g.
            # "manage_addProduct/PageTemplates/manage_addPageTemplateForm".
            # restrictedTraverse will raise Unauthorized if it
            # can't obtain the factory method by name due to a
            # security restriction.  We depend on this side effect
            # here!  Note that we use restrictedTraverse as
            # opposed to checkPermission to take into account the
            # special security circumstances related to proxy
            # roles.  See collector #78.

        except Unauthorized:
            if mt_permission:
                message = ('You do not possess the %s permission in the '
                            'context of the container into which you are '
                            'pasting, thus you are not able to perform '
                            'this operation.' % mt_permission)
            else:
                message = ('You do not possess the permission required '
                            'to call %s in the context of the container '
                            'into which you are pasting, thus you are not '
                            'able to perform this operation.' % method_name)

            raise CopyError, MessageDialog(
                title = 'Insufficient Privileges',
                message = message,
                action = 'manage_main')

        if validate_src:

            sm = getSecurityManager()

            # Ensure the user is allowed to access the object on the
            # clipboard.
            if not sm.validate(None,parent,None,object):
                raise Unauthorized, object_id

            if validate_src == 2: # moving
                if not sm.checkPermission(DeleteObjects, parent):
                    raise Unauthorized, 'Delete not allowed.'

    else: # /if method_name
        raise CopyError, MessageDialog(
                title   = 'Not Supported',
                message = ('The object <EM>%s</EM> does not support this '
                            'operation.' % object_id),
                action  = 'manage_main')

def _patched_PortalFolder_verifyObjectPaste(self, object, validate_src=1):

    # This assists the version in OFS.CopySupport.
    # It enables the clipboard to function correctly
    # with objects created by a multi-factory.
    securityChecksDone = 0
    sm = getSecurityManager()
    object_id = escape(object.getId())
    parent = aq_parent(aq_inner(object))

    mt = getattr(object, '__factory_meta_type__', None)
    meta_types = getattr(self, 'all_meta_types', None)

    if mt is not None and meta_types is not None:

        method_name = None
        permission_name = None

        if callable(meta_types):
            meta_types = meta_types()

        for d in meta_types:
            if d['name'] == mt:
                method_name = d['action']
                permission_name = d.get('permission', None)
                break

        if permission_name is not None:

            if not sm.checkPermission(permission_name, self):
                raise Unauthorized, object_id

            if not validate_src:
               # We don't want to check the object on the clipboard
               securityChecksDone = 1

            if not sm.validate(None, parent, None, object):
                raise Unauthorized, object_id

            if validate_src > 1 : # on move
                if not sm.checkPermission(DeleteObjects, parent) :
                    raise Unauthorized, object_id

            securityChecksDone = 1

        # Old validation for objects that may not have registered 
        # themselves in the proper fashion.
        #
        elif method_name is not None:

            meth = self.unrestrictedTraverse(method_name)

            try:
                factory = getattr(meth, 'im_self')
            except AttributeError:
                factory = aq_parent(aq_inner(meth))

            if not getSecurityManager().validate(None, factory, None, meth):
                raise Unauthorized, method_name

            # Ensure the user is allowed to access the object on the
            # clipboard.
            if not validate_src:
                securityChecksDone = 1

            if not sm.validate(None, parent, None, object):
                raise Unauthorized, object_id

            if validate_src > 1 : # on move
                if not sm.checkPermission(DeleteObjects, parent) :
                    raise Unauthorized, object_id

            securityChecksDone = 1

    # Call OFS' _verifyObjectPaste if necessary
    if not securityChecksDone:
        PortalFolder.inheritedAttribute(
            '_verifyObjectPaste')(self, object, validate_src)

    # Finally, check allowed content types
    contentType = getattr(object, '_getPortalTypeName', None)
    if contentType is not None:
        contentType = contentType()
        if contentType is not None:
            pt = getToolByName(self, 'portal_types')
            myType = pt.getTypeInfo(self)
            if myType is not None and not myType.allowType(contentType):
                raise ValueError, \
                        "Cannot paste subobject type '%s'." % contentType


def initialize(context):

    from OFS.CopySupport import CopyContainer
    CopyContainer.manage_pasteObjects = _patched_manage_pasteObjects
    CopyContainer._verifyObjectPaste = _patched_CopySupport_verifyObjectPaste

    if PortalFolder is not None:

        PortalFolder._verifyObjectPaste = \
            _patched_PortalFolder_verifyObjectPaste
